package es.ua.jtech.jsp.sesion11.ranking.filtro;

import java.io.IOException;
import java.io.OutputStream;
import java.sql.SQLException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import es.ua.jtech.jsp.sesion11.ranking.bd.PaginasBD;
import es.ua.jtech.jsp.sesion11.ranking.parse.AnalisisHTML;
import es.ua.jtech.jsp.sesion11.ranking.wrapper.GenericResponseWrapper;

public final class AccesoPaginaFilter implements Filter {

	private FilterConfig filterConfig = null;

	PaginasBD connection = null;

	/**
	 * Efectua el filtrado de una peticin.
	 * 
	 * @param request
	 *            Peticin realizada al servidor
	 * @param response
	 *            Respuesta para enviar al cliente
	 * @param chain
	 *            Cadena de filtros para procesar la peticin
	 */

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		HttpServletRequest http_request = null;
		HttpServletResponse http_response = null;

		if (request instanceof HttpServletRequest) {
			http_request = (HttpServletRequest) request;
		}

		if (response instanceof HttpServletResponse) {
			http_response = (HttpServletResponse) response;
		}

		// Tratamos solo peticiones HTTP

		if (http_request != null && http_response != null) {

			// Creamos un wrapper

			GenericResponseWrapper responseWrapper = new GenericResponseWrapper(
					http_response);

			// Solicitamos la URL al servidor

			chain.doFilter(http_request, responseWrapper);

			// Procesamos los datos generados

			byte[] datos = responseWrapper.getData();

			String url = http_request.getRequestURI();

			if (connection.estaRegistrada(url)) {
				// Si la pagina solicitada esta registrada, incrementa el numero
				// de accesos

				connection.incrementaAccesos(url);

			} else {
				// Si no, registra la pagina (ruta, titulo) en la base de datos

				String responseString = new String(datos);

				AnalisisHTML parser = new AnalisisHTML();

				parser.parse(responseString.toCharArray());

				String titulo = parser.getTitulo();

				connection.registraPagina(url, titulo);
			}

			// Devolvemos el contenido al cliente

			http_response.setHeader("Cache-control", "no-cache");
			http_response.setHeader("Cache-control", "no-store");
			http_response.setHeader("Pragma", "no-cache");
			http_response.setHeader("Expires", "0");
			
			OutputStream out = http_response.getOutputStream();
			out.write(datos);
			out.close();

		} else {
			chain.doFilter(request, response);
		}

	}

	/**
	 * Destruye el filtro. Cierra conexin a base de datos
	 */

	public void destroy() {
		this.filterConfig = null;

		// Cerrar conexion con base de datos
		if (connection != null) {
			try {
				connection.close();
			} catch (SQLException e) {
			}
		}

	}

	/**
	 * Inicializa el filtro. Abre conexin a BD y lee parmetros.
	 * 
	 * @param filterConfig
	 *            Configuracin del filtro y de la aplicacin.
	 */

	public void init(FilterConfig filterConfig) throws ServletException {
		this.filterConfig = filterConfig;

		// Abrir conexion con base de datos
		try {
			String urlBd = filterConfig.getServletContext().getInitParameter(
					"urlbd");
			String driverBd = filterConfig.getServletContext()
					.getInitParameter("driverbd");
			connection = new PaginasBD(driverBd, urlBd, "root", "root");
		} catch (Exception e) {
			throw new ServletException(e);
		}
	}

}
